@@ -158,7 +158,10 @@ class MySQL_Data_Stream
 	int status; // status . FIXME: make it a ORable variable
 
 	int switching_auth_stage;
-	int switching_auth_type;
+	enum proxysql_auth_plugins switching_auth_type;
+	// Updated **only** when an 'auth_switch' has been sent to client
+	enum proxysql_auth_plugins switching_auth_sent;
+	int auth_in_progress; // if 0 , no authentication is in progress. Any value greater than 0 depends from the implementation
 	unsigned int tmp_charset;
 
 	short revents;
