@@ -56,6 +56,9 @@ struct cmp_str {
 
 #define N_L_ASE 16
 
+#define AWS_ENDPOINT_SUFFIX_STRING "rds.amazonaws.com"
+#define QUERY_READ_ONLY_AND_AWS_TOPOLOGY_DISCOVERY "SELECT @@global.read_only read_only, id, endpoint, port from mysql.rds_topology"
+
 /*
 
 Implementation of monitoring in AWS Aurora will be different than previous modules
@@ -70,6 +73,14 @@ A single AWS_Aurora_monitor_node will have a AWS_Aurora_status_entry per check.
 
 */
 
+#ifdef TEST_AURORA
+
+#define TEST_AURORA_MONITOR_BASE_QUERY \
+	"SELECT SERVER_ID, SESSION_ID, LAST_UPDATE_TIMESTAMP, REPLICA_LAG_IN_MILLISECONDS, CPU"\
+		" FROM REPLICA_HOST_STATUS ORDER BY SERVER_ID "
+
+#endif
+
 class AWS_Aurora_replica_host_status_entry {
 	public:
 	char * server_id = NULL;
@@ -189,7 +200,8 @@ enum MySQL_Monitor_State_Data_Task_Type {
 	MON_GROUP_REPLICATION,
 	MON_REPLICATION_LAG,
 	MON_GALERA,
-	MON_AWS_AURORA
+	MON_AWS_AURORA,
+	MON_READ_ONLY__AND__AWS_RDS_TOPOLOGY_DISCOVERY
 };
 
 enum class MySQL_Monitor_State_Data_Task_Result {
@@ -200,6 +212,17 @@ enum class MySQL_Monitor_State_Data_Task_Result {
 	TASK_RESULT_PENDING
 };
 
+/**
+ * @brief Holds the info from a GR server definition.
+ */
+struct gr_host_def_t {
+	string host;
+	int port;
+	int use_ssl;
+	bool writer_is_also_reader;
+	int max_transactions_behind;
+	int max_transactions_behind_count;
+};
 
 class MySQL_Monitor_State_Data {
 public:
@@ -210,6 +233,7 @@ public:
 	char *hostname;
 	int port;
 	int writer_hostgroup; // used only by group replication
+	int reader_hostgroup;
 	bool writer_is_also_reader; // used only by group replication
 	int  max_transactions_behind; // used only by group replication
 	int max_transactions_behind_count; // used only by group replication
@@ -237,6 +261,11 @@ public:
 	 * @details Currently only used by 'group_replication'.
 	 */
 	uint64_t init_time = 0;
+	/**
+	 * @brief Used by GroupReplication to determine if servers reported by cluster 'members' are already monitored.
+	 * @details This way we avoid non-needed locking on 'MySQL_HostGroups_Manager' for server search.
+	 */
+	const std::vector<gr_host_def_t>* cur_monitored_gr_srvs = nullptr;
 
 	MySQL_Monitor_State_Data(MySQL_Monitor_State_Data_Task_Type task_type, char* h, int p, bool _use_ssl = 0, int g = 0);
 	~MySQL_Monitor_State_Data();
@@ -418,6 +447,7 @@ class MySQL_Monitor {
 	static bool update_dns_cache_from_mysql_conn(const MYSQL* mysql);
 	static void trigger_dns_cache_update();
 
+	void process_discovered_topology(const std::string& originating_server_hostname, const vector<MYSQL_ROW>& discovered_servers, int reader_hostgroup);
 
 	private:
 	std::vector<table_def_t *> *tables_defs_monitor;
@@ -529,7 +559,7 @@ private:
 	 * Note: Calling init_async is mandatory before executing tasks asynchronously.
 	*/
 	void monitor_ping_async(SQLite3_result* resultset);
-	void monitor_read_only_async(SQLite3_result* resultset);	
+	void monitor_read_only_async(SQLite3_result* resultset, bool do_discovery_check);
 	void monitor_replication_lag_async(SQLite3_result* resultset);
 	void monitor_group_replication_async();
 	void monitor_galera_async();
