@@ -1,3 +1,9 @@
+/**
+ * @file MySQL_Session.h
+ * @brief Declaration of the MySQL_Session class and associated types and enums.
+ */
+
+
 #ifndef __CLASS_MYSQL_SESSION_H
 #define __CLASS_MYSQL_SESSION_H
 
@@ -13,6 +19,10 @@ using json = nlohmann::json;
 
 extern class MySQL_Variables mysql_variables;
 
+/**
+ * @enum proxysql_session_type
+ * @brief Defines the types of ProxySQL sessions.
+ */
 enum proxysql_session_type {
 	PROXYSQL_SESSION_MYSQL,
 	PROXYSQL_SESSION_ADMIN,
@@ -24,11 +34,26 @@ enum proxysql_session_type {
 	PROXYSQL_SESSION_NONE
 };
 
+/**
+ * @enum ps_type
+ * @brief Defines types for prepared statement handling.
+ */
+enum ps_type : uint8_t {
+	ps_type_not_set = 0x0,
+	ps_type_prepare_stmt = 0x1,
+	ps_type_execute_stmt = 0x2
+};
+
 std::string proxysql_session_type_str(enum proxysql_session_type session_type);
 
-// these structs will be used for various regex hardcoded
-// their initial use will be for sql_log_bin , sql_mode and time_zone
-// issues #509 , #815 and #816
+/**
+ * @class Session_Regex
+ * @brief Encapsulates regex operations for session handling.
+ *
+ * This class is used for matching patterns in SQL queries, specifically for
+ * settings like sql_log_bin, sql_mode, and time_zone.
+ * See issues #509 , #815 and #816
+ */
 class Session_Regex {
 	private:
 	void *opt;
@@ -40,6 +65,13 @@ class Session_Regex {
 	bool match(char *m);
 };
 
+/**
+ * @class Query_Info
+ * @brief Holds information about a SQL query within a session.
+ *
+ * This class encapsulates various details about a query such as its text,
+ * execution times, affected rows, and more, to facilitate query processing and logging.
+ */
 class Query_Info {
 	public:
 	SQP_par_t QueryParserArgs;
@@ -78,6 +110,13 @@ class Query_Info {
 	bool is_select_NOT_for_update();
 };
 
+/**
+ * @class MySQL_Session
+ * @brief Manages a client session, including query parsing, backend connections, and state transitions.
+ *
+ * This class is central to ProxySQL's handling of client connections. It manages the lifecycle
+ * of a session, processes queries, and communicates with backend MySQL servers.
+ */
 class MySQL_Session
 {
 	private:
@@ -121,12 +160,13 @@ class MySQL_Session
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_SET_OPTION(PtrSize_t *);
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_STATISTICS(PtrSize_t *);
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_PROCESS_KILL(PtrSize_t *);
-	bool handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_QUERY_qpo(PtrSize_t *, bool *lock_hostgroup, bool ps=false);
+	bool handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_QUERY_qpo(PtrSize_t *, bool *lock_hostgroup, ps_type prepare_stmt_type=ps_type_not_set);
 
 	void handler___client_DSS_QUERY_SENT___server_DSS_NOT_INITIALIZED__get_connection();	
 
 	void return_proxysql_internal(PtrSize_t *);
 	bool handler_special_queries(PtrSize_t *);
+	bool handler_special_queries_STATUS(PtrSize_t *);
 	/**
 	 * @brief Handles 'COMMIT|ROLLBACK' commands.
 	 * @details Forwarding the packet is required when there are active transactions. Since we are limited to
@@ -140,9 +180,21 @@ class MySQL_Session
 	bool handler_CommitRollback(PtrSize_t *);
 	bool handler_SetAutocommit(PtrSize_t *);
 	/**
-	 * @brief Performs the cleanup of current session state, and the required operations to the supplied
-	 *   'MySQL_Data_Stream' required for processing further queries.
-	 * @param The 'MySQL_Data_Stream' which executed the previous query and which status should be updated.
+	 * @brief Should execute most of the commands executed when a request is finalized.
+	 * @details Cleanup of current session state, and required operations to the supplied 'MySQL_Data_Stream'
+	 *   for further queries processing. Takes care of the following actions:
+	 *   - Update the status of the backend connection (if supplied), with previous query actions.
+	 *   - Log the query for the required statuses.
+	 *   - Cleanup the previous Query_Processor output.
+	 *   - Free the resources of the backend connection (if supplied).
+	 *   - Reset all the required session status flags. E.g:
+	 *       + status
+	 *       + client_myds::DSS
+	 *       + started_sending_data_to_client
+	 *       + previous_hostgroup
+	 *   NOTE: Should become the place to hook other functions.
+	 * @param myds If not null, should point to a MySQL_Data_Stream (backend connection) which connection status
+	 *   should be updated, and previous query resources cleanup.
 	 */
 	void RequestEnd(MySQL_Data_Stream *);
 	void LogQuery(MySQL_Data_Stream *);
@@ -220,6 +272,7 @@ class MySQL_Session
 
 	Query_Info CurrentQuery;
 	PtrSize_t mirrorPkt;
+	PtrSize_t pkt;
 
 	// uint64_t
 	unsigned long long start_time;
@@ -247,7 +300,6 @@ class MySQL_Session
 
 	//this pointer is always initialized inside handler().
 	// it is an attempt to start simplifying the complexing of handler()
-	PtrSize_t *pktH;
 
 	uint32_t thread_session_id;
 	unsigned long long last_insert_id;
@@ -274,6 +326,7 @@ class MySQL_Session
 	int to_process;
 	int pending_connect;
 	enum proxysql_session_type session_type;
+	int warning_in_hg;
 
 	// bool
 	bool autocommit;
@@ -334,7 +387,7 @@ class MySQL_Session
 	MySQL_Backend * find_or_create_backend(int, MySQL_Data_Stream *_myds=NULL);
 	
 	void SQLite3_to_MySQL(SQLite3_result *, char *, int , MySQL_Protocol *, bool in_transaction=false, bool deprecate_eof_active=false);
-	void MySQL_Result_to_MySQL_wire(MYSQL *mysql, MySQL_ResultSet *MyRS, MySQL_Data_Stream *_myds=NULL);
+	void MySQL_Result_to_MySQL_wire(MYSQL *mysql, MySQL_ResultSet *MyRS, unsigned int warning_count, MySQL_Data_Stream *_myds=NULL);
 	void MySQL_Stmt_Result_to_MySQL_wire(MYSQL_STMT *stmt, MySQL_Connection *myconn);
 	unsigned int NumActiveTransactions(bool check_savpoint=false);
 	bool HasOfflineBackends();
@@ -379,7 +432,10 @@ class MySQL_Session
 	bool has_any_backend();
 	void detected_broken_connection(const char *file, unsigned int line, const char *func, const char *action, MySQL_Connection *myconn, int myerr, const char *message, bool verbose=false);
 	void generate_status_one_hostgroup(int hid, std::string& s);
+	void reset_warning_hostgroup_flag_and_release_connection();
 	friend void SQLite3_Server_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t *pkt);
+
+	void set_previous_status_mode3(bool allow_execute=true);
 };
 
 #define KILL_QUERY       1
