@@ -25,7 +25,6 @@
 #define SQLITE_HOSTGROUP -4
 
 
-#define MYSQL_DEFAULT_TX_ISOLATION	"READ-COMMITTED"
 #define MYSQL_DEFAULT_SESSION_TRACK_GTIDS      "OFF"
 #define MYSQL_DEFAULT_COLLATION_CONNECTION	""
 #define MYSQL_DEFAULT_NET_WRITE_TIMEOUT	"60"
@@ -53,32 +52,6 @@ typedef struct _kill_queue_t {
 	std::vector<thr_id_usr *> query_ids;
 } kill_queue_t;
 
-class ProxySQL_Poll {
-	private:
-	void shrink();
-	void expand(unsigned int more);
-
-	public:
-	unsigned int len;
-	unsigned int size;
-	struct pollfd *fds;
-	MySQL_Data_Stream **myds;
-	unsigned long long *last_recv;
-	unsigned long long *last_sent;
-	std::atomic<bool> bootstrapping_listeners;
-	volatile int pending_listener_add;
-	volatile int pending_listener_del;
-	unsigned int poll_timeout;
-	unsigned long loops;
-	StatCounters *loop_counters;
-
-	ProxySQL_Poll();
-	~ProxySQL_Poll();
-	void add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, unsigned long long sent_time);
-	void remove_index_fast(unsigned int i);
-	int find_index(int fd);
-};
-
 enum MySQL_Thread_status_variable {
 	st_var_backend_stmt_prepare,
 	st_var_backend_stmt_execute,
@@ -256,25 +229,6 @@ class __attribute__((aligned(64))) MySQL_Thread
 typedef MySQL_Thread * create_MySQL_Thread_t();
 typedef void destroy_MySQL_Thread_t(MySQL_Thread *);
 
-class iface_info {
-	public:
-	char *iface;
-	char *address;
-	int port;
-	int fd;
-	iface_info(char *_i, char *_a, int p, int f) {
-		iface=strdup(_i);
-		address=strdup(_a);
-		port=p;
-		fd=f;
-	}
-	~iface_info() {
-		free(iface);
-		free(address);
-		close(fd);
-	}
-};
-
 class MySQL_Listeners_Manager {
 	private:
 	PtrArray *ifaces;
@@ -350,6 +304,7 @@ struct p_th_gauge {
 		mysql_monitor_ping_interval,
 		mysql_monitor_ping_timeout,
 		mysql_monitor_ping_max_failures,
+		mysql_monitor_aws_rds_topology_discovery_interval,
 		mysql_monitor_read_only_interval,
 		mysql_monitor_read_only_timeout,
 		mysql_monitor_writer_is_also_reader,
@@ -431,6 +386,8 @@ class MySQL_Threads_Handler
 		int monitor_ping_max_failures;
 		//! Monitor ping timeout. Unit: 'ms'.
 		int monitor_ping_timeout;
+		//! Monitor aws rds topology discovery interval. Unit: 'one discovery check per X monitor_read_only checks'.
+		int monitor_aws_rds_topology_discovery_interval;
 		//! Monitor read only timeout. Unit: 'ms'.
 		int monitor_read_only_interval;
 		//! Monitor read only timeout. Unit: 'ms'.
@@ -493,8 +450,10 @@ class MySQL_Threads_Handler
 		char *interfaces;
 		char *server_version;
 		char *keep_multiplexing_variables;
+		char *default_authentication_plugin;
 		//unsigned int default_charset; // removed in 2.0.13 . Obsoleted previously using MySQL_Variables instead
 		int handle_unknown_charset;
+		int default_authentication_plugin_int;
 		bool servers_stats;
 		bool commands_stats;
 		bool query_digests;
@@ -554,7 +513,6 @@ class MySQL_Threads_Handler
 		char *init_connect;
 		char *ldap_user_variable;
 		char *add_ldap_user_comment;
-		char *default_tx_isolation;
 		char *default_session_track_gtids;
 		char *default_variables[SQL_NAME_LAST_LOW_WM];
 		char *firewall_whitelist_errormsg;
@@ -581,6 +539,7 @@ class MySQL_Threads_Handler
 		char * ssl_p2s_crlpath;
 		int query_cache_size_MB;
 		int query_cache_soft_ttl_pct;
+		int query_cache_handle_warnings;
 		int min_num_servers_lantency_awareness;
 		int aurora_max_lag_ms_only_read_from_replicas;
 		bool stats_time_backend_query;
@@ -593,6 +552,8 @@ class MySQL_Threads_Handler
 		bool enable_load_data_local_infile;
 		bool log_mysql_warnings_enabled;
 		int data_packets_history_size;
+		int handle_warnings;
+		int evaluate_replication_lag_on_servers_load;
 	} variables;
 	struct {
 		unsigned int mirror_sessions_current;
