@@ -10,36 +10,39 @@
 
 #include <openssl/x509v3.h>
 
-/*
-
-in libssl 1.1.0 
-struct bio_st {
-	const BIO_METHOD *method;
-	long (*callback) (struct bio_st *, int, const char *, int, long, long);
-	char *cb_arg;
-	int init;
-	int shutdown;
-	int flags;
-	int retry_reason;
-	int num;
-	void *ptr;
-	struct bio_st *next_bio;
-	struct bio_st *prev_bio;
-	int references;
-	uint64_t num_read;
-	uint64_t num_write;
-	CRYPTO_EX_DATA ex_data;
-	CRYPTO_RWLOCK *lock;
-};
-*/
-
-typedef int CRYPTO_REF_COUNT;
 
 /**
- * @brief This is the 'bio_st' struct definition from libssl 3.0.0. NOTE: This is an internal struct from
+ * @brief This is the 'bio_st' struct definition from libssl. NOTE: This is an internal struct from
  *   OpenSSL library, currently it's used for performing checks on the reads/writes performed on the BIO objects.
  *   It's extremely important to keep this struct up to date with each OpenSSL dependency update.
  */
+typedef int CRYPTO_REF_COUNT;
+
+#if (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x10100000
+#pragma message "libssl 1.1.x detected"
+struct bio_st {
+    const BIO_METHOD *method;
+    /* bio, mode, argp, argi, argl, ret */
+    BIO_callback_fn callback;
+    BIO_callback_fn_ex callback_ex;
+    char *cb_arg;               /* first argument for the callback */
+    int init;
+    int shutdown;
+    int flags;                  /* extra storage */
+    int retry_reason;
+    int num;
+    void *ptr;
+    struct bio_st *next_bio;    /* used by filter BIOs */
+    struct bio_st *prev_bio;    /* used by filter BIOs */
+    CRYPTO_REF_COUNT references;
+    uint64_t num_read;
+    uint64_t num_write;
+    CRYPTO_EX_DATA ex_data;
+    CRYPTO_RWLOCK *lock;
+};
+
+#elif (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x30000000 || (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x30100000
+#pragma message "libssl 3.0.x / 3.1.x detected"
 struct bio_st {
     OSSL_LIB_CTX *libctx;
     const BIO_METHOD *method;
@@ -64,6 +67,35 @@ struct bio_st {
     CRYPTO_RWLOCK *lock;
 };
 
+#elif (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x30200000
+#pragma message "libssl 3.2.x detected"
+struct bio_st {
+    OSSL_LIB_CTX *libctx;
+    const BIO_METHOD *method;
+    /* bio, mode, argp, argi, argl, ret */
+#ifndef OPENSSL_NO_DEPRECATED_3_0
+    BIO_callback_fn callback;
+#endif
+    BIO_callback_fn_ex callback_ex;
+    char *cb_arg;               /* first argument for the callback */
+    int init;
+    int shutdown;
+    int flags;                  /* extra storage */
+    int retry_reason;
+    int num;
+    void *ptr;
+    struct bio_st *next_bio;    /* used by filter BIOs */
+    struct bio_st *prev_bio;    /* used by filter BIOs */
+    CRYPTO_REF_COUNT references;
+    uint64_t num_read;
+    uint64_t num_write;
+    CRYPTO_EX_DATA ex_data;
+};
+
+#else
+#error "libssl version not supported: OPENSSL_VERSION_NUMBER = " ##OPENSSL_VERSION_NUMBER
+#endif
+
 
 #define RESULTSET_BUFLEN_DS_16K 16000
 #define RESULTSET_BUFLEN_DS_1M 1000*1024
@@ -305,7 +337,9 @@ MySQL_Data_Stream::MySQL_Data_Stream() {
 	DSS=STATE_NOT_CONNECTED;
 	encrypted=false;
 	switching_auth_stage = 0;
-	switching_auth_type = 0;
+	switching_auth_type = AUTH_UNKNOWN_PLUGIN;
+	switching_auth_sent = AUTH_UNKNOWN_PLUGIN;
+	auth_in_progress = 0;
 	x509_subject_alt_name=NULL;
 	ssl=NULL;
 	rbio_ssl = NULL;
@@ -736,7 +770,12 @@ int MySQL_Data_Stream::write_to_net() {
 					memmove(ssl_write_buf, ssl_write_buf+n, ssl_write_len-n);
 				}
 				ssl_write_len -= n;
-    			ssl_write_buf = (char*)realloc(ssl_write_buf, ssl_write_len);
+				if (ssl_write_len) {
+					ssl_write_buf = (char*)realloc(ssl_write_buf, ssl_write_len);
+				} else {
+					free(ssl_write_buf);
+					ssl_write_buf = NULL;
+				}
 				//proxy_info("new ssl_write_len: %u\n", ssl_write_len);
 				//if (ssl_write_len) {
     			//	return n; // stop here
@@ -887,7 +926,12 @@ int MySQL_Data_Stream::write_to_net_poll() {
 					memmove(ssl_write_buf, ssl_write_buf+n, ssl_write_len-n);
 				}
 				ssl_write_len -= n;
-    			ssl_write_buf = (char*)realloc(ssl_write_buf, ssl_write_len);
+				if (ssl_write_len) {
+					ssl_write_buf = (char*)realloc(ssl_write_buf, ssl_write_len);
+				} else {
+					free(ssl_write_buf);
+					ssl_write_buf = NULL;
+				}
 				//proxy_info("new ssl_write_len: %u\n", ssl_write_len);
 				if (ssl_write_len) {
     				return n; // stop here
@@ -1284,8 +1328,16 @@ int MySQL_Data_Stream::array2buffer() {
 					memcpy(&queueOUT.pkt,PSarrayOUT->index(idx), sizeof(PtrSize_t));
 					idx++;
 		//VALGRIND_ENABLE_ERROR_REPORTING;
-					// this is a special case, needed because compression is enabled *after* the first OK
-					if (DSS==STATE_CLIENT_AUTH_OK) {
+					// This is a special case, needed because compression is enabled *after* the first OK. In
+					// case of 'caching_sha2_password', not only the first packet needs to be processed, since
+					// there are other scenarios in which one extra byte is sent prior to the final OK packet
+					// flagging auth success. The generation of these extra packets should all be queued at
+					// the same time, since they represent the final client response. Right now this is
+					// handled during 'MySQL_Session::handler___status_CONNECTING_CLIENT___STATE_SERVER_HANDSHAKE'.
+					// Because of this, we can make the assumption that once we have sent all the packets
+					// currently in 'PSarrayOUT', it's safe to change the 'DSS' status, and enable compression
+					// if connections requires it.
+					if (DSS==STATE_CLIENT_AUTH_OK && idx == PSarrayOUT->len) {
 						DSS=STATE_SLEEP;
 						// enable compression
 						if (myconn->options.server_capabilities & CLIENT_COMPRESS) {
