@@ -116,9 +116,6 @@ get_sources(){
     git clone ${GIT_REPO}
     cd proxysql-packaging
     git checkout ${GIT_BRANCH}
-    sed -i 's/proxysql-login-files/proxysql-login-file/' debian/rules
-    sed -i 's/proxysql-login-files/proxysql-login-file/' debian/rules.systemd
-    sed -i 's/proxysql-login-files/proxysql-login-file/' debian/rules.xenial || true
     cd ../
     git clone ${PAT_REPO}
     cd proxysql-admin-tool
@@ -140,14 +137,13 @@ get_sources(){
     git checkout ${PROXYSQL_BRANCH}
     GIT_VERSION=${VERSION}-percona
     REVISION=$(git rev-parse --short HEAD)
+    sed -i "s/export CURVER?=.*/export CURVER?=${VERSION}/g" Makefile
     sed -i 's/shell cat.*/shell rpm --eval \%rhel)/' deps/Makefile
     sed -i 's:6.7:6:' deps/Makefile
     sed -i 's/shell cat.*/shell rpm --eval \%rhel)/' src/Makefile
     sed -i 's:6.7:6:' src/Makefile
     sed -i '94s|exit 0||' etc/init.d/proxysql
     sed -i "s:GITVERSION:\"${GIT_VERSION}-${RPM_RELEASE}\":g" include/proxysql.h
-    sed -i 's/2.6.0/2.5.5/' Makefile
-    grep CURVER Makefile
     cd ..
     echo "REVISION=${REVISION}" >> ${WORKDIR}/proxysql.properties
     cd ${PRODUCT_FULL}
@@ -371,7 +367,7 @@ install_deps() {
           PATH=$PATH:/usr/local/bin
           cmake --version
       fi
-      if [ "x${DEBIAN}" = "xxenial" -o "x${DEBIAN}" = "xbionic" ]; then
+      if [ "x${DEBIAN}" = "xbuster" -o "x${DEBIAN}" = "xbionic" ]; then
 	  wget https://repo.percona.com/percona/apt/percona-release_latest.${DEBIAN}_all.deb
           dpkg -i ./percona-release_latest.${DEBIAN}_all.deb
           percona-release enable tools testing
