@@ -31,6 +31,151 @@ using std::vector;
 using std::to_string;
 using nlohmann::json;
 
+#define LAST_QUERY_EXECUTED_STR(mysql)	(*static_cast<std::string*>(mysql->unused_0)) 
+#define STMT_VECTOR(stmt)				(*static_cast<std::vector<MYSQL_STMT*>*>(stmt->mysql->unused_3))
+#define STMT_EXECUTED_VECTOR(stmt)		(*static_cast<std::vector<std::unique_ptr<char,decltype(&free)>>*>(stmt->mysql->unused_4))
+#define LAST_QUERY_EXECUTED_PTR(mysql)	(static_cast<std::string*>(mysql->unused_0))
+#define STMT_VECTOR_PTR(mysql)			(static_cast<std::vector<MYSQL_STMT*>*>(mysql->unused_3))
+#define STMT_EXECUTED_VECTOR_PTR(mysql)	(static_cast<std::vector<std::unique_ptr<char,decltype(&free)>>*>(mysql->unused_4))
+
+#define STMT_FIND_INDEX(stmt,idx)		const std::vector<MYSQL_STMT*>& vec_stmt = STMT_VECTOR(stmt); \
+										for (size_t i = 0; i < vec_stmt.size(); i++) {\
+											if (vec_stmt[i] == stmt) {\
+												idx = i; \
+												break; \
+											}\
+										}	
+
+#define STMT_PUSH_QUERY(stmt,query)		size_t idx = -1; \
+										STMT_FIND_INDEX(stmt,idx);\
+										if (idx == -1) {\
+											STMT_VECTOR(stmt).emplace_back(stmt); \
+											STMT_EXECUTED_VECTOR(stmt).emplace_back(strdup(query), &free);\
+										} else {\
+											STMT_EXECUTED_VECTOR(stmt)[idx] = std::unique_ptr<char,decltype(&free)>(strdup(query), &free);\
+										}
+
+#define STMT_LOAD_QUERY(stmt,query)		size_t idx = -1; \
+										STMT_FIND_INDEX(stmt,idx);\
+										if (idx != -1) query = STMT_EXECUTED_VECTOR(stmt)[idx].get();
+
+#define STMT_REMOVE(stmt)				size_t idx = -1; \
+										STMT_FIND_INDEX(stmt,idx);\
+										if (idx != -1) {\
+											std::vector<MYSQL_STMT*>& vec_stmt = STMT_VECTOR(stmt);\
+											std::vector<std::unique_ptr<char,decltype(&free)>>& vec_query = STMT_EXECUTED_VECTOR(stmt);\
+											if (idx != vec_stmt.size() - 1) {\
+												vec_stmt[idx] = vec_stmt.back();\
+												vec_query[idx] = std::move(vec_query.back());\
+											}\
+											vec_stmt.pop_back();\
+											vec_query.pop_back();\
+										}
+
+MYSQL* mysql_init_override(MYSQL* mysql, const char* file, int line) {
+	static bool init = false;
+	MYSQL* result = (*real_mysql_init)(mysql);
+	if (init == false) {
+		init = true;
+		fprintf(stdout, ">> [mysql_init] Override functions attached <<\n");
+	}
+	result->unused_0 = new std::string;
+	result->unused_3 = nullptr;
+	result->unused_4 = nullptr;
+	return result;
+}
+
+int mysql_query_override(MYSQL* mysql, const char* query, const char* file, int line) {
+	const int result = (*real_mysql_query)(mysql, query);
+	if (result == 0) {
+		LAST_QUERY_EXECUTED_STR(mysql) = query;
+		if (mysql_errno(mysql) == 0 && mysql_field_count(mysql) == 0 && mysql_warning_count(mysql) > 0) {
+			fprintf(stdout, "File %s, Line %d, [mysql_query] A warning was generated during the execution of the query:'%s', warning count:%d\n",
+				file, line, query, mysql_warning_count(mysql));
+		}
+	}
+	return result;
+}
+
+MYSQL_RES* mysql_store_result_override(MYSQL* mysql, const char* file, int line) {
+	MYSQL_RES* result = (*real_mysql_store_result)(mysql);
+	if (mysql_errno(mysql) == 0 && mysql_warning_count(mysql) > 0) {
+		fprintf(stdout, "File %s, Line %d, [mysql_store_result] A warning was generated during the execution of the query:'%s', warning count:%d\n",
+			file, line, LAST_QUERY_EXECUTED_STR(mysql).c_str(), mysql_warning_count(mysql));
+	}
+	return result;
+}
+
+void mysql_close_override(MYSQL* mysql, const char* file, int line) {
+	delete LAST_QUERY_EXECUTED_PTR(mysql);
+	if (STMT_VECTOR_PTR(mysql)) {
+		delete STMT_VECTOR_PTR(mysql);
+		delete STMT_EXECUTED_VECTOR_PTR(mysql);
+	}
+	(*real_mysql_close)(mysql);
+}
+
+MYSQL_STMT* mysql_stmt_init_override(MYSQL* mysql, const char* file, int line) {
+	MYSQL_STMT* result = (*real_mysql_stmt_init)(mysql);
+	if (result->mysql->unused_3 == nullptr) {
+		std::vector<MYSQL_STMT*>* vec_stmt = new std::vector<MYSQL_STMT*>;
+		std::vector<std::unique_ptr<char,decltype(&free)>>* vec_query = 
+			new std::vector<std::unique_ptr<char,decltype(&free)>>;
+		vec_stmt->reserve(64);
+		vec_query->reserve(64);
+		result->mysql->unused_3 = vec_stmt;
+		result->mysql->unused_4 = vec_query;
+	}
+	return result;
+}
+
+int mysql_stmt_prepare_override(MYSQL_STMT* stmt, const char* stmt_str, unsigned long length, const char* file, int line) {
+	const int result = (*real_mysql_stmt_prepare)(stmt, stmt_str, length);
+	if (result == 0) {
+		STMT_PUSH_QUERY(stmt,stmt_str);
+		// mysql_stmt_warning_count is not available in MySQL connector
+		if (mysql_stmt_errno(stmt) == 0 && /*mysql_stmt_warning_count(stmt)*/mysql_warning_count(stmt->mysql) > 0) {
+			fprintf(stdout, "File %s, Line %d, [mysql_stmt_prepare] A warning was generated during the execution of the query:'%s', warning count:%d\n",
+				file, line, stmt_str, /*mysql_stmt_warning_count(stmt)*/mysql_warning_count(stmt->mysql));
+		}
+	}
+	return result;
+}
+
+int mysql_stmt_execute_override(MYSQL_STMT* stmt, const char* file, int line) {
+	const int result = (*real_mysql_stmt_execute)(stmt);
+	if (result == 0) {
+		// mysql_stmt_warning_count is not available in MySQL connector
+		if (mysql_stmt_errno(stmt) == 0 && mysql_stmt_field_count(stmt) == 0 && 
+			/*mysql_stmt_warning_count(stmt)*/mysql_warning_count(stmt->mysql) > 0) {
+			char* query = nullptr;
+			STMT_LOAD_QUERY(stmt, query);
+			fprintf(stdout, "File %s, Line %d, [mysql_stmt_execute] A warning was generated during the execution of the query:'%s', warning count:%d\n",
+				file, line, (query ? query : ""), /*mysql_stmt_warning_count(stmt)*/mysql_warning_count(stmt->mysql));
+		}
+	}
+	return result;
+}
+
+int mysql_stmt_store_result_override(MYSQL_STMT* stmt, const char* file, int line) {
+	const int result = (*real_mysql_stmt_store_result)(stmt);
+	if (result == 0) {
+		// mysql_stmt_warning_count is not available in MySQL connector
+		if (mysql_stmt_errno(stmt) == 0 && /*mysql_stmt_warning_count(stmt)*/mysql_warning_count(stmt->mysql) > 0) {
+			char* query = nullptr;
+			STMT_LOAD_QUERY(stmt, query);
+			fprintf(stdout, "File %s, Line %d, [mysql_stmt_store_result] A warning was generated during the execution of the query:'%s', warning count:%d\n",
+				file, line, (query ? query : ""), /*mysql_stmt_warning_count(stmt)*/mysql_warning_count(stmt->mysql));
+		}
+	}
+	return result;
+}
+
+my_bool mysql_stmt_close_override(MYSQL_STMT* stmt, const char* file, int line) {
+	STMT_REMOVE(stmt)
+	return (*real_mysql_stmt_close)(stmt);
+}
+
 std::size_t count_matches(const string& str, const string& substr) {
 	std::size_t result = 0;
 	std::size_t pos = 0;
@@ -278,19 +423,19 @@ int execvp(const string& cmd, const std::vector<const char*>& argv, string& resu
 
 		if (err == 0) {
 			// Read from child’s stdout
-			count = read(PARENT_READ_FD, buffer, sizeof(buffer));
+			count = read(PARENT_READ_FD, buffer, sizeof(buffer)-1);
 			while (count > 0) {
 				buffer[count] = 0;
 				result_ += buffer;
-				count = read(PARENT_READ_FD, buffer, sizeof(buffer));
+				count = read(PARENT_READ_FD, buffer, sizeof(buffer)-1);
 			}
 		} else {
 			// Read from child’s stderr
-			count = read(PARENT_READ_ERR, buffer, sizeof(buffer));
+			count = read(PARENT_READ_ERR, buffer, sizeof(buffer)-1);
 			while (count > 0) {
 				buffer[count] = 0;
 				result_ += buffer;
-				count = read(PARENT_READ_ERR, buffer, sizeof(buffer));
+				count = read(PARENT_READ_ERR, buffer, sizeof(buffer)-1);
 			}
 		}
 
@@ -352,6 +497,63 @@ std::vector<mysql_res_row> extract_mysql_rows(MYSQL_RES* my_res) {
 	return result;
 };
 
+pair<uint32_t,vector<mysql_res_row>> mysql_query_ext_rows(MYSQL* mysql, const string& query) {
+	int rc = mysql_query(mysql, query.c_str());
+	if (rc != EXIT_SUCCESS) {
+		return { mysql_errno(mysql), {} };
+	}
+
+	MYSQL_RES* myres = mysql_store_result(mysql);
+	if (myres == nullptr) {
+		return { mysql_errno(mysql), {} };
+	}
+
+	const vector<mysql_res_row> rows { extract_mysql_rows(myres) };
+	mysql_free_result(myres);
+
+	return { EXIT_SUCCESS, rows };
+}
+
+ext_val_t<string> ext_single_row_val(const mysql_res_row& row, const string& def_val) {
+	if (row.empty() || row.front().empty()) {
+		return { -1, def_val, {} };
+	} else {
+		return { EXIT_SUCCESS, string { row[0] }, string { row[0] } };
+	}
+}
+
+ext_val_t<int64_t> ext_single_row_val(const mysql_res_row& row, const int64_t& def_val) {
+	if (row.empty() || row.front().empty()) {
+		return { -1, def_val, {} };
+	} else {
+        errno = 0;
+        char* p_end {};
+        const int64_t val = std::strtoll(row.front().c_str(), &p_end, 10);
+
+		if (row[0] == p_end || errno == ERANGE) {
+			return { -2, def_val, string { row[0] } };
+		} else {
+			return { EXIT_SUCCESS, val, string { row[0] } };
+		}
+	}
+}
+
+ext_val_t<uint64_t> ext_single_row_val(const mysql_res_row& row, const uint64_t& def_val) {
+	if (row.empty() || row.front().empty()) {
+		return { -1, def_val, {} };
+	} else {
+        errno = 0;
+        char* p_end {};
+        const uint64_t val = std::strtoll(row.front().c_str(), &p_end, 10);
+
+		if (row[0] == p_end || errno == ERANGE) {
+			return { -2, def_val, string { row[0] } };
+		} else {
+			return { EXIT_SUCCESS, val, string { row[0] } };
+		}
+	}
+}
+
 struct memory {
 	char* data;
 	size_t size;
@@ -595,17 +797,6 @@ int wait_for_replication(
 	return result;
 }
 
-MARIADB_CHARSET_INFO * proxysqlTap_find_charset_collate(const char *collatename) {
-	MARIADB_CHARSET_INFO *c = (MARIADB_CHARSET_INFO *)mariadb_compiled_charsets;
-	do {
-		if (!strcasecmp(c->name, collatename)) {
-			return c;
-		}
-		++c;
-	} while (c[0].nr != 0);
-	return NULL;
-}
-
 int create_proxysql_user(
 	MYSQL* proxysql_admin,
 	const string& user,
@@ -944,51 +1135,19 @@ cleanup:
 }
 
 int get_cur_backend_conns(MYSQL* proxy_admin, const string& conn_type, uint32_t& found_conn_num) {
-	MYSQL_QUERY(proxy_admin, string {"SELECT " + conn_type + " FROM stats_mysql_connection_pool"}.c_str());
+	MYSQL_QUERY(proxy_admin, string {"SELECT SUM(" + conn_type + ") FROM stats_mysql_connection_pool"}.c_str());
 
 	MYSQL_ROW row = nullptr;
-	MYSQL_RES* my_res = mysql_store_result(proxy_admin);
-	uint32_t field_num = mysql_num_fields(my_res);
-	vector<uint32_t> connfree_vals {};
+	MYSQL_RES* myres = mysql_store_result(proxy_admin);
+	uint32_t field_num = mysql_num_fields(myres);
 
 	if (field_num != 1) {
 		diag("Invalid number of columns in resulset from 'stats_mysql_connection_pool': %d", field_num);
-	}
-
-	if (my_res != nullptr) {
-		while ((row = mysql_fetch_row(my_res))) {
-			connfree_vals.push_back(std::strtol(row[0], NULL, 10));
-		}
-	}
-	mysql_free_result(my_res);
-
-	found_conn_num = std::accumulate(connfree_vals.begin(), connfree_vals.end(), 0, std::plus<uint32_t>());
-	return EXIT_SUCCESS;
-}
-
-int wait_for_backend_conns(
-	MYSQL* proxy_admin, const string& conn_type, uint32_t exp_conn_num, uint32_t timeout
-) {
-	uint32_t total_conn_num = 0;
-	uint32_t waited = 0;
-
-	while (waited < timeout) {
-		int get_err = get_cur_backend_conns(proxy_admin, conn_type, total_conn_num);
-		if (get_err != EXIT_SUCCESS) { return EXIT_FAILURE; }
-
-		if (total_conn_num == exp_conn_num) {
-			break;
-		} else {
-			sleep(1);
-			waited += 1;
-		}
-	}
-
-	if (waited >= timeout) {
-		return EXIT_FAILURE;
 	} else {
-		return EXIT_SUCCESS;
+		found_conn_num = std::strtol(row[0], NULL, 10);
 	}
+
+	return EXIT_SUCCESS;
 }
 
 string join_path(const string& p1, const string& p2) {
@@ -1579,3 +1738,53 @@ void check_query_count(MYSQL* admin, vector<uint32_t> queries, uint32_t hg) {
 		dump_conn_stats(admin, { hg });
 	}
 };
+
+const char* get_env_str(const char* envname, const char* envdefault) {
+
+	const char* envval = std::getenv(envname);
+
+	if (envval != NULL)
+		return envval;
+
+	return envdefault;
+};
+
+int get_env_int(const char* envname, int envdefault) {
+
+	const char* envval = std::getenv(envname);
+	int res = envdefault;
+
+	if (envval != NULL)
+		res = strtol(envval, NULL, 0);
+//	diag("%s: %s='%s' >>> %d", __FUNCTION__, envname, envval, res);
+
+	return res;
+};
+
+bool get_env_bool(const char* envname, bool envdefault) {
+
+	const char* envval = std::getenv(envname);
+	int res = envdefault;
+
+	if (envval != NULL) {
+		if (strcasecmp(envval, "true") == 0) {
+			res = 1;
+		} else if (strcasecmp(envval, "false") == 0) {
+			res = 0;
+		} else if (strcasecmp(envval, "yes") == 0) {
+			res = 1;
+		} else if (strcasecmp(envval, "no") == 0) {
+			res = 0;
+		} else if (strcasecmp(envval, "on") == 0) {
+			res = 1;
+		} else if (strcasecmp(envval, "off") == 0) {
+			res = 0;
+		} else {
+			res = strtol(envval, NULL, 0);
+		}
+	}
+
+//	diag("%s: %s='%s' >>> %d", __FUNCTION__, envname, envval, res);
+
+	return (bool) res;
+};
