@@ -21,8 +21,8 @@ It stresses how proxysql managers prepared statements, specifically:
 #include <tuple>
 #include <unistd.h>
 
-#include <mysql.h>
-#include <mysql/mysqld_error.h>
+#include "mysql.h"
+#include "mysqld_error.h"
 
 #include "command_line.h"
 #include "proxysql_utils.h"
