@@ -11,7 +11,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <vector>
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
