@@ -13,7 +13,7 @@
 #include <vector>
 #include <string>
 #include <stdio.h>
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
