@@ -18,15 +18,14 @@
 #include <string>
 #include <stdio.h>
 
-#include <mysql.h>
-#include <mysql/mysqld_error.h>
+#include "mysql.h"
+#include "mysqld_error.h"
 
-#include <proxysql_utils.h>
+#include "proxysql_utils.h"
 
 #include "tap.h"
 #include "command_line.h"
 #include "utils.h"
-#include "utils.h"
 
 using std::vector;
 using std::tuple;
