@@ -28,8 +28,8 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include <mysql.h>
-#include <mysql/mysqld_error.h>
+#include "mysql.h"
+#include "mysqld_error.h"
 
 #include "tap.h"
 #include "command_line.h"
@@ -51,9 +51,17 @@ std::vector<std::pair<std::string,std::string>> credentials = {
 	{"cliuser4", "clipass4"}
 };
 
+int set_clickhouse_host(MYSQL *pa, const char *h) {
+	std::string query = "SET clickhouse-host=" + std::string(h);
+	diag("Line: %d . Setting clickhouse-host to '%s'", __LINE__ , h);
+	MYSQL_QUERY(pa, query.c_str()); 
+	MYSQL_QUERY(pa, "LOAD CLICKHOUSE VARIABLES TO RUNTIME");
+	return 0;
+}
+
 int set_clickhouse_port(MYSQL *pa, int p) {
 	std::string query = "SET clickhouse-port=" + std::to_string(p);
-	diag("Line: %d . Setting clickhouse-port to %d", __LINE__ , p);
+	diag("Line: %d . Setting clickhouse-port to '%d'", __LINE__ , p);
 	MYSQL_QUERY(pa, query.c_str()); 
 	MYSQL_QUERY(pa, "LOAD CLICKHOUSE VARIABLES TO RUNTIME");
 	return 0;
@@ -403,10 +411,17 @@ int main(int argc, char** argv) {
 			goto cleanup;
 		}
 
-		set_clickhouse_port(proxysql_admin,8000);
+		set_clickhouse_port(proxysql_admin, 8000);
 		test_crash(host_port.first.c_str(), host_port.second);
-		set_clickhouse_port(proxysql_admin,19000);
-//		set_clickhouse_port(proxysql_admin,9000);
+
+		const std::string docker_mode = getenv("DOCKER_MODE");
+		if (docker_mode.find("dns") == docker_mode.size() - 3) {
+			set_clickhouse_host(proxysql_admin, "clickhouse");
+			set_clickhouse_port(proxysql_admin, 9000);
+		} else {
+			set_clickhouse_host(proxysql_admin, "127.0.0.1");
+			set_clickhouse_port(proxysql_admin, 19000);
+		}
 
 		MYSQL* proxysql_clickhouse = mysql_init(NULL);
 
