@@ -5,7 +5,7 @@
 #include <unistd.h>
 
 #include <string>
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
