@@ -4,7 +4,7 @@
 #include <vector>
 #include <string>
 
-#include <mysql.h>
+#include "mysql.h"
 #include "tap.h"
 #include "command_line.h"
 #include "utils.h"
