@@ -17,13 +17,14 @@
 #include <iostream>
 
 #include <mysql.h>
-#include <mysql/mysqld_error.h>
+#include "mysqld_error.h"
 
 #include "json.hpp"
 #include "tap.h"
 #include "command_line.h"
 #include "utils.h"
 
+
 using nlohmann::json;
 
 /**
